#!/usr/bin/env python3

from . import cli
from .. import Workspace
from pprint import pprint

@cli.priority(30)
def new():
    """\
    Create a new experiment with a blank notebook entry.

    Usage:
        exmemo [note] new <title>

    Arguments:
        <title>
            The title of the experiment.  This should be title cased, with 
            words separated by spaces.  Use quotes so the shell won't interpret 
            it as multiple arguments.
    """
    args = cli.parse_args_via_docopt()
    work = Workspace.from_cwd()

    work.init_experiment(args['<title>'])

@cli.priority(30)
def edit():
    """\
    Open the notebook entry for the given experiment in a text editor.

    Usage:
        exmemo [note] edit [<substr>]

    Arguments:
        <substr>
            The experiment to edit.  You don't need to specify the whole name, 
            just enough to be unique.  If multiple experiments match the given 
            substr, you'll will be asked which one you mean.  If you don't 
            specify an experiment, the most recently created one will be used.

    You can specify which text editor you prefer in an `.exmemorc` file, or via 
    the $EDITOR environment variable.
    """
    args = cli.parse_args_via_docopt()
    work = Workspace.from_cwd()
    expt = work.pick_experiment(args['<substr>'])
    path = work.get_notebook_entry(expt)

    work.launch_editor(path)

@cli.priority(30)
def open():
    """\
    Open a new terminal cd'd into the given experiment.

    Usage:
        exmemo [note] open [<substr>]

    Arguments:
        <substr>
            The experiment to open.  You don't need to specify the whole name, 
            just enough to be unique.  If multiple experiments match the given 
            substr, you'll will be asked which one you mean.  If you don't 
            specify an experiment, the most recently created one will be used.

    You can specify which terminal you prefer in an `.exmemorc` file, or via 
    the $TERMINAL environment variable.
    """
    args = cli.parse_args_via_docopt()
    work = Workspace.from_cwd()
    expt = work.pick_experiment(args['<substr>'])

    work.launch_terminal(expt)

@cli.priority(30)
def build():
    """\
    Render the lab notebook to HTML using Sphinx.

    Usage:
        exmemo [note] build [-f]

    Options:
        -f --force
            Force Sphinx to rebuild the whole project from scratch, rather than 
            just building the files that changed since last time.  You often 
            need to do this after you add a new experiment.
    """
    args = cli.parse_args_via_docopt()
    work = Workspace.from_cwd()

    work.build_notebook(args['--force'])

def ls():
    """\
    Print the names of any existing experiments.

    Usage:
        exmemo note ls [<substr>]

    Arguments:
        <substr>
            Only print the experiments matching the given text.
    """
    args = cli.parse_args_via_docopt()
    workspace = Workspace.from_cwd()

    for expt in workspace.iter_experiments(args['<substr>']):
        print(expt.name)

